'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _ramda = require('ramda');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isErrorEmpty = (0, _ramda.anyPass)([_ramda.isNil, _ramda.isEmpty, (0, _ramda.complement)(Boolean)]);

var capitalize = function capitalize(str) {
  return str && '' + str[0].toUpperCase() + str.slice(1);
};

var getValue = function getValue(event) {
  if (event.target) {
    return (0, _ramda.contains)(event.target.value, ['on', 'off']) ? event.target.checked : event.target.value;
  }

  return event;
};

var isCheckable = function isCheckable(element) {
  return element.props.type === 'radio' || element.props.type === 'checkbox' || typeof element.props.checked !== 'undefined' || typeof (element.type.defaultProps || {}).checked !== 'undefined';
};

var Form = function (_Component) {
  _inherits(Form, _Component);

  function Form(props) {
    _classCallCheck(this, Form);

    var _this = _possibleConstructorReturn(this, (Form.__proto__ || Object.getPrototypeOf(Form)).call(this, props));

    _this.state = {
      data: props.data || {},
      errors: props.errors || {}
    };

    _this.cloneTree = _this.cloneTree.bind(_this);
    _this.validateTree = _this.validateTree.bind(_this);
    _this.validateElement = _this.validateElement.bind(_this);
    _this.notifyChangeEvent = _this.notifyChangeEvent.bind(_this);
    _this.handleEvent = _this.handleEvent.bind(_this);
    _this.handleSubmit = _this.handleSubmit.bind(_this);
    _this.mergeErrors = _this.mergeErrors.bind(_this);

    if (props.validateDataProp) {
      _this.state.errors = _this.validateTree(_this.state.errors, _this);
    }
    return _this;
  }

  _createClass(Form, [{
    key: 'mergeErrors',
    value: function mergeErrors(newErrors) {
      if (newErrors && !(0, _ramda.equals)(newErrors, this.state.errors)) {
        this.setState({ errors: (0, _ramda.merge)(this.state.errors, newErrors) });
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var _this2 = this;

      var data = nextProps.data,
          nextErrors = nextProps.errors;


      if (data && !(0, _ramda.equals)(data, this.props.data)) {
        this.setState({ data: data }, function () {
          if (_this2.props.validateDataProp) {
            var errors = _this2.validateTree(_this2.state.errors, _this2);

            if (nextErrors && !(0, _ramda.equals)(errors, nextErrors)) {
              _this2.setState({ errors: (0, _ramda.merge)(errors, nextErrors) });
              return;
            }
            _this2.setState({ errors: errors });
            return;
          }
          _this2.mergeErrors(nextErrors);
        });
        return;
      }

      this.mergeErrors(nextErrors);
    }
  }, {
    key: 'notifyChangeEvent',
    value: function notifyChangeEvent() {
      var onChange = this.props.onChange;


      if (typeof onChange === 'function') {
        var _state = this.state,
            data = _state.data,
            errors = _state.errors;

        onChange(data, errors);
      }
    }
  }, {
    key: 'validateElement',
    value: function validateElement(path, data, errors) {
      var lens = (0, _ramda.lensPath)(path);
      var validation = (0, _ramda.view)(lens, this.props.validation);

      if (!validation) {
        return errors;
      }

      var value = (0, _ramda.view)(lens, data);

      if ((0, _ramda.is)(Array, validation)) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = validation[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var validate = _step.value;

            var err = validate(value);

            if (!isErrorEmpty(err)) {
              return (0, _ramda.set)(lens, err, errors);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      } else if (typeof validation === 'function') {
        var _err = validation(value);

        if (!isErrorEmpty(_err)) {
          return (0, _ramda.set)(lens, _err, errors);
        }
      }

      var parentPath = (0, _ramda.dropLast)(1, path);

      if (parentPath.length > 0) {
        if ((0, _ramda.pathSatisfies)(isErrorEmpty, parentPath, errors)) {
          return (0, _ramda.dissocPath)(parentPath, errors);
        }
      }

      return (0, _ramda.dissocPath)(path, errors);
    }
  }, {
    key: 'handleEvent',
    value: function handleEvent(eventName, path, originalHandler, event) {
      if (typeof originalHandler === 'function') {
        originalHandler(event);
      }

      if (event && event.defaultPrevented) {
        return;
      }

      var validateOn = this.props.validateOn;


      var data = this.state.data;
      var errors = this.state.errors;

      if (eventName === 'change') {
        var lens = (0, _ramda.lensPath)(path);
        var value = getValue(event);

        data = (0, _ramda.set)(lens, value, this.state.data);
      }

      if (eventName === validateOn) {
        errors = this.validateElement(path, data, errors);
      }

      this.setState({ data: data, errors: errors }, this.notifyChangeEvent);
    }
  }, {
    key: 'cloneTree',
    value: function cloneTree(element, index) {
      var parentPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

      if (!element || typeof element === 'string') {
        return element;
      }

      var path = element.props.name ? [].concat(_toConsumableArray(parentPath), [element.props.name]) : parentPath;

      if (element.type === 'fieldset') {
        return _react2.default.cloneElement(element, {}, _react2.default.Children.map(element.props.children, (0, _ramda.partialRight)(this.cloneTree, [path])));
      }

      if (element.props.role === 'alert' && element.props.htmlFor) {
        var lens = (0, _ramda.lensPath)([].concat(_toConsumableArray(path), [element.props.htmlFor]));
        var errors = (0, _ramda.view)(lens, this.state.errors);

        if (errors) {
          var children = (0, _ramda.is)(Array, errors) ? errors[0] : errors;
          return _react2.default.cloneElement(element, { children: children });
        }

        return element;
      }

      if (element.props.name) {
        var _lens = (0, _ramda.lensPath)(path);

        var _props = this.props,
            validateOn = _props.validateOn,
            _props$customErrorPro = _props.customErrorProp,
            errorProp = _props$customErrorPro === undefined ? 'error' : _props$customErrorPro;


        var props = {};

        props.onChange = (0, _ramda.partial)(this.handleEvent, ['change', path, element.props.onChange]);

        if (['blur', 'focus'].includes(validateOn)) {
          var eventProp = 'on' + capitalize(validateOn);

          props[eventProp] = (0, _ramda.partial)(this.handleEvent, [validateOn, path, element.props[eventProp]]);
        }

        var value = (0, _ramda.view)(_lens, this.state.data);

        if (isCheckable(element)) {
          if (typeof value === 'boolean') {
            props.checked = value;
          } else {
            props.checked = value === element.props.value;
          }
        } else {
          props.value = value;
        }

        var error = (0, _ramda.view)(_lens, this.state.errors);

        if (error) {
          props[errorProp] = error;
        }

        return _react2.default.cloneElement(element, props);
      }

      if (element.props.children) {
        return _react2.default.cloneElement(element, {}, _react2.default.Children.map(element.props.children, (0, _ramda.partialRight)(this.cloneTree, [path])));
      }

      return element;
    }
  }, {
    key: 'validateTree',
    value: function validateTree() {
      var errors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var element = arguments[1];
      var parentPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

      if (!element || typeof element === 'string') {
        return errors;
      }

      var children = _react2.default.Children.toArray(element.props.children);
      var path = element.props.name ? [].concat(_toConsumableArray(parentPath), [element.props.name]) : parentPath;

      if (element.type !== 'select' && children.length > 0) {
        var childErrors = (0, _ramda.reduce)((0, _ramda.partialRight)(this.validateTree, [path]), errors, children);

        if (isErrorEmpty(childErrors)) {
          var _parentPath = (0, _ramda.dropLast)(1, path);

          if (_parentPath.length === 0) {
            return {};
          }

          return (0, _ramda.dissocPath)(_parentPath, errors);
        }

        return childErrors;
      }

      if (element.props.name) {
        return this.validateElement(path, this.state.data, errors);
      }

      return errors;
    }
  }, {
    key: 'handleSubmit',
    value: function handleSubmit(event) {
      var _this3 = this;

      event.preventDefault();

      var errors = this.validateTree({}, this);

      this.setState({ errors: errors }, function () {
        return isErrorEmpty(_this3.state.errors) ? _this3.props.onSubmit(_this3.state.data) : _this3.props.onSubmit(_this3.state.data, _this3.state.errors);
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var className = this.props.className;


      return _react2.default.createElement(
        'form',
        { onSubmit: this.handleSubmit, className: className },
        _react2.default.Children.map(this.props.children, (0, _ramda.partialRight)(this.cloneTree, [this, []]))
      );
    }
  }]);

  return Form;
}(_react.Component);

exports.default = Form;


Form.propTypes = {
  /**
   * The children can contain any kind of component. Inputs with name
   * property will be tracked for changes using `onChange` callback.
   * Sibling labels with `role=alert` and `htmlFor` pointing to a validated
   * component will be used to present the error message.
  **/
  children: _propTypes.node,
  /**
   * The validation object whose keys mirror form field structure.
   * Values of this object can be either functions or a function array.
   * Validation functions receives the input string and should return
   * a string message on error and a falsy value otherwise.
  **/
  validation: _propTypes.object, // eslint-disable-line
  /**
  * The event where validation will be triggered. By default, field
  * validations runs on `change` event.
  **/
  validateOn: (0, _propTypes.oneOf)(['change', 'blur', 'focus']),
  /**
   * The form submit callback. Receives the serialized form as an object.
  **/
  onSubmit: _propTypes.func,
  /**
   * The form change callback. This callback runs on every form control's
   * `onChange`, right after validations. When this is defined, the form
   * behaves as a controlled component, and the user is responsible for
   * updating the form state via `data` prop.
  **/
  /**
   * @callback onChange
   * @param {object} data
   * @param {object} errors
  **/
  onChange: _propTypes.func,
  /**
   * The form data object whose keys mirror form field structure.
   * Setting this prop will set the form controls' values accordingly.
   * This can be used for rendering an initial state or to use the form
   * as a controlled component.
  **/
  data: _propTypes.object, // eslint-disable-line
  /**
   * A property name to inject the error message in the named field.
   * This is useful for input wrappers with builtin error label, commonly
   * found in UI libraries.
  **/
  customErrorProp: _propTypes.string,
  className: _propTypes.string
};

Form.defaultProps = {
  children: undefined,
  className: '',
  customErrorProp: undefined,
  data: undefined,
  onChange: undefined,
  onSubmit: undefined,
  validateDataProp: false,
  validateOn: 'change',
  validation: {}
};
